<?php

namespace edu\wisc\services\cbs\order\payment;

use edu\wisc\services\cbs\common\Order;
use Money\Money;

/**
 * OrderPayment represents payment information needed by CBS for processing an order.
 */
class OrderPayment extends Order
{

    /** @var float */
    private $orderNumber;

    /** @var string */
    private $origSysDocumentRef;

    /** @var string Approval Code (check #, CC approval) */
    private $paymentInfo;

    /** @var string Credit Card or DNumber */
    private $paymentType;

    /** @var string IP of register used for sale */
    private $register;

    /** @var Money Amount of payment */
    private $renderedAmount;

    /** @var Money */
    private $subtotal;

    /** @var Money */
    private $tax;

    /** @var Money */
    private $total;

    /** @var string (optional) */
    private $paymentNotes;

    /** @var string Fixed for Magento orders */
    private $salesRep = Order::SALES_REP;

    /** @var string Fixed for Magento orders */
    private $orderSource = Order::ORDER_SOURCE;

    /** @var string */
    private $testMode;

    /**
     * @return float
     */
    public function getOrderNumber(): float
    {
        return $this->orderNumber;
    }

    /**
     * @param float $orderNumber
     * @return OrderPayment
     */
    public function setOrderNumber(float $orderNumber): OrderPayment
    {
        $this->orderNumber = $orderNumber;
        return $this;
    }

    /**
     * @return string
     */
    public function getOrigSysDocumentRef(): string
    {
        return $this->origSysDocumentRef;
    }

    /**
     * @param string $origSysDocumentRef
     * @return OrderPayment
     */
    public function setOrigSysDocumentRef(string $origSysDocumentRef): OrderPayment
    {
        $this->origSysDocumentRef = $origSysDocumentRef;
        return $this;
    }

    /**
     * @return string
     */
    public function getPaymentInfo(): string
    {
        return $this->paymentInfo;
    }

    /**
     * @param string $paymentInfo
     * @return OrderPayment
     */
    public function setPaymentInfo(string $paymentInfo): OrderPayment
    {
        $this->paymentInfo = $paymentInfo;
        return $this;
    }

    /**
     * @return string
     */
    public function getPaymentType(): string
    {
        return $this->paymentType;
    }

    /**
     * @param string $paymentType
     * @return OrderPayment
     */
    public function setPaymentType(string $paymentType): OrderPayment
    {
        $this->paymentType = $paymentType;
        return $this;
    }

    /**
     * @return string
     */
    public function getRegister(): string
    {
        return $this->register;
    }

    /**
     * @param string $register
     * @return OrderPayment
     */
    public function setRegister(string $register): OrderPayment
    {
        $this->register = $register;
        return $this;
    }

    /**
     * @return Money
     */
    public function getRenderedAmount(): Money
    {
        return $this->renderedAmount;
    }

    /**
     * @param Money $renderedAmount
     * @return OrderPayment
     */
    public function setRenderedAmount(Money $renderedAmount): OrderPayment
    {
        $this->renderedAmount = $renderedAmount;
        return $this;
    }

    /**
     * @return Money
     */
    public function getSubtotal(): Money
    {
        return $this->subtotal;
    }

    /**
     * @param Money $subtotal
     * @return OrderPayment
     */
    public function setSubtotal(Money $subtotal): OrderPayment
    {
        $this->subtotal = $subtotal;
        return $this;
    }

    /**
     * @return Money
     */
    public function getTax(): Money
    {
        return $this->tax;
    }

    /**
     * @param Money $tax
     * @return OrderPayment
     */
    public function setTax(Money $tax): OrderPayment
    {
        $this->tax = $tax;
        return $this;
    }

    /**
     * @return Money
     */
    public function getTotal(): Money
    {
        return $this->total;
    }

    /**
     * @param Money $total
     * @return OrderPayment
     */
    public function setTotal(Money $total): OrderPayment
    {
        $this->total = $total;
        return $this;
    }

    /**
     * @return string|null
     */
    public function getPaymentNotes()
    {
        return $this->paymentNotes;
    }

    /**
     * @param string|null $paymentNotes
     * @return OrderPayment
     */
    public function setPaymentNotes(string $paymentNotes = null): OrderPayment
    {
        $this->paymentNotes = $paymentNotes;
        return $this;
    }

    /**
     * @return string
     */
    public function getSalesRep(): string
    {
        return $this->salesRep;
    }

    /**
     * @param string $salesRep
     * @return OrderPayment
     */
    public function setSalesRep(string $salesRep): OrderPayment
    {
        $this->salesRep = $salesRep;
        return $this;
    }

    /**
     * @return string
     */
    public function getOrderSource(): string
    {
        return $this->orderSource;
    }

    /**
     * @param string $orderSource
     * @return OrderPayment
     */
    public function setOrderSource(string $orderSource): OrderPayment
    {
        $this->orderSource = $orderSource;
        return $this;
    }

    /**
     * @return string|null
     */
    public function getTestMode()
    {
        return $this->testMode;
    }

    /**
     * @param string|null $testMode
     * @return OrderPayment
     */
    public function setTestMode(string $testMode = null): OrderPayment
    {
        $this->testMode = $testMode;
        return $this;
    }

}